package com.iusmob.ad_klein_flutter_sdk;

import android.app.Activity;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.VisibleForTesting;

import java.util.HashMap;
import java.util.Map;

import com.iusmob.adklein.ad.AdKleinInitConfig;
import com.iusmob.adklein.ad.AdKleinSDK;

import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.StandardMethodCodec;
import io.flutter.plugin.platform.PlatformViewRegistry;

/**
 * AdKleinFlutterSdkPlugin
 */
public class AdKleinFlutterSdkPlugin implements FlutterPlugin, ActivityAware,
        MethodChannel.MethodCallHandler {

    @Nullable
    private FlutterPluginBinding pluginBinding;
    @Nullable
    private AdInstanceManager instanceManager;

    private final Map<String, NativeAdFactory> nativeAdFactories = new HashMap<>();

    public static boolean registerNativeAdFactory(FlutterEngine engine, String factoryId) {
        final AdKleinFlutterSdkPlugin gmaPlugin =
                (AdKleinFlutterSdkPlugin) engine.getPlugins().get(AdKleinFlutterSdkPlugin.class);
        return registerNativeAdFactory(gmaPlugin, factoryId);
    }

    private static boolean registerNativeAdFactory(AdKleinFlutterSdkPlugin plugin, String factoryId) {
        if (plugin == null) {
            final String message =
                    String.format(
                            "Could not find a %s instance. The plugin may have not been " +
                                    "registered.",
                            AdKleinFlutterSdkPlugin.class.getSimpleName());
            throw new IllegalStateException(message);
        }

        return plugin.addNativeAdFactory(factoryId);
    }

    private boolean addNativeAdFactory(String factoryId) {
        if (nativeAdFactories.containsKey(factoryId)) {
            final String errorMessage =
                    String.format(
                            "A NativeAdFactory with the following factoryId already exists: %s",
                            factoryId);
            Log.e(AdKleinFlutterSdkPlugin.class.getSimpleName(), errorMessage);
            return false;
        }

        nativeAdFactories.put(factoryId, null);
        return true;
    }

    /**
     * Public constructor for the plugin. Dependency initialization is handled in lifecycle methods
     * below.
     */
    public AdKleinFlutterSdkPlugin() {
    }

    /**
     * Constructor for testing.
     */
    @VisibleForTesting
    protected AdKleinFlutterSdkPlugin(
            @Nullable FlutterPluginBinding pluginBinding,
            @Nullable AdInstanceManager instanceManager) {
        this.pluginBinding = pluginBinding;
        this.instanceManager = instanceManager;
    }

    public interface NativeAdFactory {

    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        pluginBinding = binding;
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {

    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        initializePlugin(
                binding.getActivity(),
                pluginBinding.getBinaryMessenger(),
                pluginBinding.getPlatformViewRegistry());
    }

    private void initializePlugin(Activity activity, BinaryMessenger messenger,
                                  PlatformViewRegistry platformViewRegistry) {
        final MethodChannel channel =
                new MethodChannel(
                        messenger,
                        "ad_klein_flutter_sdk",
                        new StandardMethodCodec(new AdMessageCodec()));
        channel.setMethodCallHandler(this);
        instanceManager = new AdInstanceManager(activity, messenger, channel);

        pluginBinding.getPlatformViewRegistry().registerViewFactory("ADKleinAdView",
                new ADKleinAdViewFactory(instanceManager));
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        switch (call.method) {
            case "initSdk":
                // Internal init. This is necessary to cleanup state on hot restart.
                instanceManager.disposeAllAds();
                // 初始化ADKlein广告SDK
                AdKleinSDK.init(instanceManager.activity, new AdKleinInitConfig.Builder()
                        .mediaId(call.arguments()+"")
                        .deBug(BuildConfig.DEBUG)
                        .build());
                result.success(null);
                break;
            case "loadSplashAd":
                final FlutterSplashAd splashAd =
                        new FlutterSplashAd.Builder()
                                .setManager(instanceManager)
                                .setAdPosId(call.<String>argument("posId"))
                                .build();
                instanceManager.trackAd(splashAd, call.<Integer>argument("adId"));
                splashAd.load();
                result.success(null);
                break;
            case "loadBannerAd":
                final FlutterBannerAd bannerAd =
                        new FlutterBannerAd.Builder()
                                .setManager(instanceManager)
                                .setAdPosId(call.<String>argument("posId"))
                                .setAdWidth(call.<Double>argument("adWidth"))
                                .setAdHeight(call.<Double>argument("adHeight"))
                                .build();
                instanceManager.trackAd(bannerAd, call.<Integer>argument("adId"));
                bannerAd.load();
                result.success(null);
                break;
            case "loadNativeAd":
                final FlutterNativeAd nativeAd =
                        new FlutterNativeAd.Builder()
                                .setManager(instanceManager)
                                .setAdPosId(call.<String>argument("posId"))
                                .build();
                instanceManager.trackAd(nativeAd, call.<Integer>argument("adId"));
                nativeAd.load();
                result.success(null);
                break;
            case "loadRewardAd":
                final FlutterRewardVodAd rewardVodAd =
                        new FlutterRewardVodAd.Builder()
                                .setManager(instanceManager)
                                .setAdPosId(call.<String>argument("posId"))
                                .build();
                instanceManager.trackAd(rewardVodAd, call.<Integer>argument("adId"));
                rewardVodAd.load();
                result.success(null);
                break;
            case "loadFullScreenVodAd":
                final FlutterFullScreenVodAd fullScreenVodAd =
                        new FlutterFullScreenVodAd.Builder()
                                .setManager(instanceManager)
                                .setAdPosId(call.<String>argument("posId"))
                                .build();
                instanceManager.trackAd(fullScreenVodAd, call.<Integer>argument("adId"));
                fullScreenVodAd.load();
                result.success(null);
                break;
            case "loadIntertitialAd":
                final FlutterInterstitialAd interstitialAd =
                        new FlutterInterstitialAd.Builder()
                                .setManager(instanceManager)
                                .setAdPosId(call.<String>argument("posId"))
                                .build();
                instanceManager.trackAd(interstitialAd, call.<Integer>argument("adId"));
                interstitialAd.load();
                result.success(null);
                break;
            case "showIntertitialAd":
            case "showRewardAd":
            case "showFullScreenVodAd":
                FlutterAd flutterFullScreenVodAd = instanceManager.adForId(call.<Integer>argument
                        ("adId"));
                if (flutterFullScreenVodAd instanceof FlutterAd.FlutterOverlayAd) {
                    ((FlutterAd.FlutterOverlayAd) flutterFullScreenVodAd).show();
                }
                result.success(null);
                break;
        }
    }

}
